package core

import (
	"math"

	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/expression"
	"git.urbach.dev/cli/q/src/types"
)

var _len = Function{OutputTypes: []types.Type{types.AnyInt}}

// CompileLen returns the length of a slice.
func (f *Function) CompileLen(root *expression.Expression) error {
	value, err := f.Evaluate(root.Children[1])

	if err != nil {
		return err
	}

	if !types.Is(value.Type, types.AnyArray) {
		return errors.New(&errors.TypeMismatch{Encountered: value.Type.Name(), Expected: types.AnyArray.Name(), ParameterName: "array"}, f.File, root.Children[1].Token.Position)
	}

	f.SaveRegister(f.CPU.Output[0])
	f.MemoryRegister(asm.LOAD, asm.Memory{Base: value.Register, Offset: -8, OffsetRegister: math.MaxUint8, Length: 8}, f.CPU.Output[0])
	f.FreeRegister(value.Register)
	return nil
}
