package core

import (
	"git.urbach.dev/cli/q/src/ast"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/expression"
	"git.urbach.dev/cli/q/src/scope"
	"git.urbach.dev/cli/q/src/token"
)

// Evaluate evaluates an expression and returns a register that contains the value of the expression.
func (f *Function) Evaluate(expr *expression.Expression) (scope.Value, error) {
	if expr.Token.Kind == token.Identifier {
		name := expr.Token.Text(f.File.Bytes)
		variable := f.VariableByName(name)

		if variable == nil {
			return scope.Value{}, errors.New(&errors.UnknownIdentifier{Name: name}, f.File, expr.Token.Position)
		}

		if variable.Alive == 1 {
			f.UseVariable(variable)
			return variable.Value, nil
		}
	}

	if ast.IsFunctionCall(expr) {
		types, err := f.CompileCall(expr)

		if err != nil {
			return scope.Value{}, err
		}

		value := scope.Value{
			Type:     types[0],
			Register: f.CPU.Output[0],
			Alive:    1,
		}

		return value, nil
	}

	tmp := f.NewRegister()
	typ, err := f.ExpressionToRegister(expr, tmp)

	value := scope.Value{
		Type:     typ,
		Register: tmp,
		Alive:    1,
	}

	return value, err
}
